(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.HowMuch = factory());
}(this, function () { 'use strict';

  var babelHelpers = {};

  babelHelpers.classCallCheck = function (instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  };

  babelHelpers.createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  babelHelpers.inherits = function (subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  };

  babelHelpers.possibleConstructorReturn = function (self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  };

  babelHelpers;

  var utils = {
      round: function round(val) {
          var places = arguments.length <= 1 || arguments[1] === undefined ? 2 : arguments[1];
          return Number(val.toFixed(places));
      }
  };

  var TaxBandCalculator = function () {

      /**
       * Instantiate the Class
       *
       * @param  {Object}  options
       */

      function TaxBandCalculator(options) {
          babelHelpers.classCallCheck(this, TaxBandCalculator);

          this.opts = options;
      }

      /**
       * Get the gross annual income
       *
       * @return  {Number}
       */


      babelHelpers.createClass(TaxBandCalculator, [{
          key: 'net',


          /**
           * Get the net annual income
           *
           * @return  {Number}
           */
          value: function net() {
              return this.gross - this.annual();
          }

          /**
           * Get the total taxable amount from the gross amount.
           * Defaults to the total gross amount.
           *
           * @return {Number}
           */

      }, {
          key: 'getTotalTaxable',
          value: function getTotalTaxable() {
              return this.gross;
          }

          /**
           * Calculate how much tax is to be paid for a particular tax band
           *
           * @param  {Number}  amount
           * @param  {Object}  band
           *
           * @return {Number}
           */

      }, {
          key: 'calculateTaxForBand',
          value: function calculateTaxForBand(amount, band) {
              if (amount < band.min) return 0;

              // return the flat rate if one is set
              if (typeof band.flat !== 'undefined') return band.rate;

              return (amount - band.min) * band.rate;
          }

          /**
           * Calculate the amount of annual Tax given the gross income
           *
           * @return {Number}
           */

      }, {
          key: 'annual',
          value: function annual() {
              var total = 0,
                  taxable = this.getTotalTaxable();

              for (var i = this.opts.bands.length - 1; i >= 0; i--) {
                  var band = this.opts.bands[i],
                      amount = this.calculateTaxForBand(taxable, band);

                  total += amount;

                  if (amount > 0) taxable = band.min;
              }

              return utils.round(total);
          }

          /**
           * Calculate the amount of monthly Tax given the gross income
           *
           * @return  {Number}
           */

      }, {
          key: 'month',
          value: function month() {
              return utils.round(this.annual() / 12);
          }

          /**
           * Calculate the amount of weekly Tax given the gross income
           *
           * @return  {Number}
           */

      }, {
          key: 'week',
          value: function week() {
              return utils.round(this.annual() / 52);
          }

          /**
           * Calculate the amount of daily Tax given the gross income
           *
           * @return  {Number}
           */

      }, {
          key: 'day',
          value: function day() {
              return utils.round(this.week() / 7);
          }
      }, {
          key: 'gross',
          get: function get() {
              return this.opts.gross;
          }

          /**
           * Get the gross annual income
           *
           * @param  {Number}  amount
           */
          ,
          set: function set(amount) {
              this.opts.gross = amount;
          }
      }]);
      return TaxBandCalculator;
  }();

  var IncomeTaxCalculator = function (_TaxBandCalculator) {
      babelHelpers.inherits(IncomeTaxCalculator, _TaxBandCalculator);


      /**
       * Instantiate the Class
       *
       * @param  {Number}  gross
       * @param  {Object}  data
       */

      function IncomeTaxCalculator(gross, data) {
          babelHelpers.classCallCheck(this, IncomeTaxCalculator);

          var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(IncomeTaxCalculator).call(this, { gross: gross, bands: data.INCOME_TAX_BANDS }));

          _this.gross = gross;
          _this.data = data;
          return _this;
      }

      /**
       * Override the total taxable amount for income tax specifically
       *
       * @return {Number}
       */


      babelHelpers.createClass(IncomeTaxCalculator, [{
          key: 'getTotalTaxable',
          value: function getTotalTaxable() {
              if (this.gross < this.data.TAX_FREE_ALLOWANCE_LIMIT) {
                  var taxable = this.gross - this.data.TAX_FREE_ALLOWANCE;

                  return taxable < 0 ? 0 : taxable;
              } else if (this.gross < this.data.TAX_FREE_ALLOWANCE_UPPER_LIMIT) {
                  var amountOver = this.gross - this.data.TAX_FREE_ALLOWANCE_LIMIT,
                      adjustedTFA = this.data.TAX_FREE_ALLOWANCE - amountOver * this.data.TAX_FREE_ALLOWANCE_LIMIT_RATE;

                  return this.gross - adjustedTFA;
              }

              return this.gross;
          }
      }]);
      return IncomeTaxCalculator;
  }(TaxBandCalculator);

  var NationalInsuranceCalculator = function (_TaxBandCalculator) {
      babelHelpers.inherits(NationalInsuranceCalculator, _TaxBandCalculator);


      /**
       * Instantiate the Class
       *
       * @param  {Number}   gross
       * @param  {Boolean}  selfEmployed
       * @param  {Object}   data
       */

      function NationalInsuranceCalculator(gross) {
          var selfEmployed = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];
          var data = arguments[2];
          babelHelpers.classCallCheck(this, NationalInsuranceCalculator);

          var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(NationalInsuranceCalculator).call(this, { gross: gross, bands: selfEmployed ? data.NI_BANDS_SE : data.NI_BANDS }));

          _this.gross = gross;
          return _this;
      }

      return NationalInsuranceCalculator;
  }(TaxBandCalculator);

  var incomeTaxData = {

    /**
     * The value that can be earned before paying any tax
     *
     * @type {Integer}
     */
    TAX_FREE_ALLOWANCE: 10600,

    /**
     * The amount at which the tax free allowance is reduced
     *
     * @type {Integer}
     */
    TAX_FREE_ALLOWANCE_LIMIT: 100000,

    /**
     * The amount at which the tax free allowance is removed
     *
     * @type {Integer}
     */
    TAX_FREE_ALLOWANCE_UPPER_LIMIT: 121200,

    /**
     * The rate to apply to the tax free allowance when it reaches the
     * threshold
     *
     * @type {Number}
     */
    TAX_FREE_ALLOWANCE_LIMIT_RATE: 0.5,

    /**
     * The various tax bands
     *
     * @type {Array}
     */
    INCOME_TAX_BANDS: [{ min: 0, rate: 0.2 }, // Basic rate
    { min: 31786, rate: 0.4 }, // Higher Rate
    { min: 150000, rate: 0.45 } // Additional Rate
    ]
  };

  var nationalInsuranceData = {

      /**
       * Define the rates for Class 1 contributions
       *
       * @type {Array}
       */
      NI_BANDS: [{ min: 0, rate: 0 }, { min: 8060, rate: 0.12 }, { min: 42386, rate: 0.02 }],

      /**
       * Define the rates for Class 2 and 4 contributions
       *
       * @type {Array}
       */
      NI_BANDS_SE: [{ min: 0, rate: 0 }, { min: 5965, rate: 145.6, flat: true }, // Class 2 flat rate
      { min: 8060, rate: 0.09 }, // Class 4 onwards
      { min: 42386, rate: 0.02 }]
  };

  var data = {
      incomeTaxData: incomeTaxData,
      nationalInsuranceData: nationalInsuranceData
  };

  var Tax = function () {

      /**
       * Instantiate the Class
       *
       * @param  {Object}  options
       */

      function Tax(options) {
          babelHelpers.classCallCheck(this, Tax);

          this.opts = options;
          this.getNet = false;
      }

      /**
       * Return the IncomeTaxCalculator.
       *
       * @return  {IncomeTaxCalculator}
       */


      babelHelpers.createClass(Tax, [{
          key: 'incomeTax',
          value: function incomeTax() {
              return new IncomeTaxCalculator(this.opts.amount, data.incomeTaxData);
          }

          /**
           * Return the NationalInsuranceCalculator.
           *
           * @return  {NationalInsuranceCalculator}
           */

      }, {
          key: 'nationalInsurance',
          value: function nationalInsurance() {
              return new NationalInsuranceCalculator(this.opts.amount, this.opts.selfEmployed, data.nationalInsuranceData);
          }

          /**
           * Get the totoal combined tax to be deducted.
           *
           * @return  {Number}
           */

      }, {
          key: 'totalTax',
          value: function totalTax() {
              return this.incomeTax().annual() + this.nationalInsurance().annual();
          }

          /**
           * Get the net amount after tax deductions.
           *
           * @return  {Number}
           */

      }, {
          key: 'calcNet',
          value: function calcNet() {
              return this.opts.amount - this.totalTax();
          }

          /**
           * Update to return net instead of tax
           *
           * @return  {Tax}
           */

      }, {
          key: 'net',
          value: function net() {
              var getNet = arguments.length <= 0 || arguments[0] === undefined ? true : arguments[0];

              this.getNet = getNet;

              return this;
          }

          /**
           * Calculate the amount of tax to be deducted, or the net income after deductions.
           *
           * @return  {Number}
           */

      }, {
          key: 'calculate',
          value: function calculate() {
              return utils.round(this.getNet ? this.calcNet() : this.totalTax());
          }

          /**
           * Calculate the annual tax/income.
           *
           * @return  {Number}
           */

      }, {
          key: 'annual',
          value: function annual() {
              return utils.round(this.calculate());
          }

          /**
           * Calculate the monthly tax/income.
           *
           * @return  {Number}
           */

      }, {
          key: 'month',
          value: function month() {
              return utils.round(this.annual() / 12);
          }

          /**
           * Calculate the weekly tax/income.
           *
           * @return  {Number}
           */

      }, {
          key: 'week',
          value: function week() {
              return utils.round(this.annual() / 52);
          }

          /**
           * Calculate the daily tax/income.
           *
           * @return  {Number}
           */

      }, {
          key: 'day',
          value: function day() {
              return utils.round(this.week() / 7);
          }

          /**
           * Get the yield of the net income vs gross income.
           *
           * @return  {[type]}
           */

      }, {
          key: 'percentage',
          value: function percentage() {
              return utils.round(this.calcNet() / this.opts.amount * 100);
          }
      }]);
      return Tax;
  }();

  var InterestCalculator = function () {

      /**
       * Instantiate the Class
       *
       * @param  {Object} options
       */

      function InterestCalculator(options) {
          babelHelpers.classCallCheck(this, InterestCalculator);

          this.opts = options;
      }

      /**
       * Get the interest rate
       *
       * @return {Number}
       */


      babelHelpers.createClass(InterestCalculator, [{
          key: 'getMonthlyInterestRate',


          /**
           * Get the monthly interest rate
           *
           * @return {Number}
           */
          value: function getMonthlyInterestRate() {
              return this.rate / 1200;
          }

          /**
           * Calculate the monthly repayment amount
           *
           * @return {Number}
           */

      }, {
          key: 'monthlyRepayment',
          value: function monthlyRepayment() {
              var rate = this.getMonthlyInterestRate();

              return utils.round(this.amount * rate / (1 - Math.pow(1 / (1 + rate), this.term)));
          }

          /**
           * Calculate the total amount payable including interest
           *
           * @return {Number}
           */

      }, {
          key: 'totalRepayment',
          value: function totalRepayment() {
              return utils.round(this.monthlyRepayment() * this.term);
          }

          /**
           * Calculate the total amount of interest payable
           *
           * @return {Number}
           */

      }, {
          key: 'totalInterest',
          value: function totalInterest() {
              return utils.round(this.totalRepayment() - this.amount);
          }
      }, {
          key: 'rate',
          get: function get() {
              return this.opts.rate;
          }

          /**
           * Get the term
           *
           * @return {Number}
           */
          ,


          /**
           * Set the interest rate
           *
           * @param  {Number} rate
           */
          set: function set(rate) {
              this.opts.rate = rate;
          }

          /**
           * Set the payment term
           *
           * @param  {Number} term
           */

      }, {
          key: 'term',
          get: function get() {
              return this.opts.term;
          }

          /**
           * Get the amount subject to interest
           *
           * @return {Number}
           */
          ,
          set: function set(term) {
              this.opts.term = term;
          }

          /**
           * Set the amount
           *
           * @param  {Number} amount
           */

      }, {
          key: 'amount',
          get: function get() {
              return this.opts.amount;
          },
          set: function set(amount) {
              this.opts.amount = amount;
          }
      }]);
      return InterestCalculator;
  }();

  var HowMuch = function () {
      function HowMuch() {
          babelHelpers.classCallCheck(this, HowMuch);
      }

      babelHelpers.createClass(HowMuch, null, [{
          key: 'incomeTax',


          /**
           * Get an instance of the IncomeTaxCalculator.
           *
           * @param   {Object}  options
           *
           * @return  {IncomeTaxCalculator}
           */
          value: function incomeTax(options) {
              return new Tax(options).incomeTax();
          }

          /**
           * Get an instance of the NationalInsuranceCalculator.
           *
           * @param   {Object}  options
           *
           * @return  {NationalInsuranceCalculator}
           */

      }, {
          key: 'nationalInsurance',
          value: function nationalInsurance(options) {
              return new Tax(options).nationalInsurance();
          }

          /**
           * Get an instance of the Tax class to calculate overall tax alone.
           *
           * @param   {Object}  options
           *
           * @return  {Tax}
           */

      }, {
          key: 'tax',
          value: function tax(options) {
              return new Tax(options);
          }

          /**
           * Get an instance of the Tax class to calculate net
           * amount after tax deductions.
           *
           * @param   {Object}  options
           *
           * @return  {Tax}
           */

      }, {
          key: 'income',
          value: function income(options) {
              return new Tax(options).net();
          }

          /**
           * Get an instance of the InterestCalculator
           *
           * @param   {Object}  options
           *
           * @return  {InterestCalculator}
           */

      }, {
          key: 'interest',
          value: function interest(options) {
              return new InterestCalculator(options);
          }
      }]);
      return HowMuch;
  }();

  return HowMuch;

}));
//# sourceMappingURL=howmuch.js.map